//+--------------------------------------------------------------------------+
//|                               ChartSpec                                  |
//|                                                                          |
//|     Example using the OpenGL 3D chart primitives built into GDImage.     |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                                                                          |
//|                           copyright (c) 2013                             |
//|                                                                          |
//|                Patrice Terrier http://www.zapsolution.com                |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 12-24-2005 (MM-DD-YYYY)            |
//|                        Last revised : 10-29-2013 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

#include "stdafx.h"
#include <windows.h>
#include <Commctrl.h>
#include <vector>
#include <string>
#include <time.h>
#include <gl/GL.h>
#include <Uxtheme.h>

//#include "zTrace.h"   // Search in the same directory of the current file.
#pragma comment(lib, "D:\\VS2010\\GDImage64.lib")
#pragma comment(lib, "D:\\VS2010\\GDImage32.lib")
#include "GDImage.h"

#pragma comment(lib, "D:\\VS2010\\WinLIFT64.lib")
#pragma comment(lib, "D:\\VS2010\\WinLIFT32.lib")
#include "WinLIFT.h"  // Search in the same directory of the current file.

#define GWL_WNDPROC         (-4)

#define LOINT(a) ((SHORT)(a))
#define HIINT(a) ((SHORT)(((DWORD)(a) >> 16) & 0xFFFF))
#define UBOUND(T) (T.size())
#define long_proc typedef long (__stdcall *zProc)
#define strSize(s) ( sizeof(s) / sizeof(s[0]) ) 

#define GRID_WIDTH     30
#define GRID_HEIGHT    9
#define TAIL_LENGTH    6
#define MAX_PARTICLES  500

struct TVertex {
    float X;
    float Y;
    float Z;
};

struct TParticle {
    long NextVertex;
    long DestValue;
    long Visited[TAIL_LENGTH];
    float Speed;
    float Step;
};

struct ZCHART {
    long chartnumber;
    long chartseries;
    float radius;
    long listindex;
};

struct THISOBJECT {
    float radius;
    float barsize;
    float xoff;
    float yoff;
    float zoff;
    float xrot;
    float yrot;
    float xcpy;
    float ycpy;
    long  move;
    long  xdown;
    long  ydown;
    long  mouseDown;
    long  listIndex;
};

#define ClientW         868
#define ClientH         532

#define IDC_CTRL       -100
#define IDC_STATIC     -101
#define IDC_LABEL      -102
#define IDC_CLOCK      -103
#define IDC_LOGO       -104
#define IDC_SPINNER    -105

#define ID_SPIN01      1

#define GL_MULTISAMPLE_ARB 0x809D

vector <ZGLTEXTUREX> g_mt;
vector <TParticle> g_Particle;
vector <TVertex> g_Grid;

//THISOBJECT g_GLobj;
WNDPROC g_hGLproc;
HWND gh_GLhWnd;
float gr_Percent;
long gn_Particles;
wstring g_sresource;
long gn_GRID_WIDTH2;

long zTrace (IN wstring sPtr) {
    long nRet = 0;
    static HMODULE hDll;
    if (hDll == 0) { 
        if (sizeof(LONG_PTR) == 8) {
            hDll = LoadLibrary(L"zTrace64"); }
        else {
            hDll = LoadLibrary(L"zTrace32");
        }
    }
    if (hDll) {
        long_proc (WCHAR*);
        zProc hProc = (zProc) GetProcAddress(hDll, "zTrace");
        if (hProc) { nRet = hProc((WCHAR*) sPtr.c_str()); }
    }
    return nRet;
}

wstring EXEpath() {
    wstring sResult;
    WCHAR exepath[MAX_PATH] = {0};
    if (GetModuleFileName(0, exepath, sizeof(exepath))) {
       sResult = exepath;
       sResult = sResult.substr( 0, sResult.rfind(L"\\"));
       sResult += L"\\";
    }
    return sResult;
}

wstring STRL$(IN long N) {
    WCHAR longstr[33] = {0};
    _ltow_s(N, longstr, 10);
    return (WCHAR*) longstr;
}

wstring STRF$(IN float N) {
    WCHAR ws[33] = {0};
    swprintf(ws, strSize(ws), L"%f", N);
    return (WCHAR*) ws;
}

// Main window minimum horizontal track size
long MinTrackSizeW (IN long SizeX) { // dllexport
    static long WasSizeX;
    if (SizeX) { WasSizeX = SizeX; }
    return WasSizeX;
}

// Main window minimum vertical track size
long MinTrackSizeH (IN long SizeY) { // dllexport
    static long WasSizeY;
    if (SizeY) { WasSizeY = SizeY; }
    return WasSizeY;
}

void InitObject() {

    gn_GRID_WIDTH2 = GRID_WIDTH * GRID_WIDTH;

    float rXoff = 0.0f;   // Initial x location.
    float rYoff = 0.0f;   // Initial y location.
    float rZoff = -31.0f; // Initial zoom factor.

    float rZoom = 1.0f;

    float rXrot = 5.0f;
    float rYrot = 0.0f;

    // User variables
    gn_Particles = 150;
    g_Grid.resize(gn_GRID_WIDTH2 * GRID_HEIGHT + 1);
    g_Particle.resize(MAX_PARTICLES + 1);

    GL_ChartSetMousing(rXoff, rYoff, rZoff, rZoom, rXrot, rYrot);
}

void InitParticle() {
    long I, J;
    // Initialise all particles
    for (I = 1; I <= MAX_PARTICLES; I++) {
        g_Particle[I].Visited[0] = (rand() % (gn_GRID_WIDTH2 * GRID_HEIGHT - 1));
        for (J = 1; J < TAIL_LENGTH; J++) {
            g_Particle[I].Visited[J] = g_Particle[I].Visited[0];
        }
        g_Particle[I].NextVertex = g_Particle[I].Visited[0];
        g_Particle[I].Speed = 0.01f + (rand() % 5) / 100.0f;
        g_Particle[I].DestValue = 0;
    }

    // initialise g_Grid
    for (I = 0; I < gn_GRID_WIDTH2 * GRID_HEIGHT; I++) {
        g_Grid[I].X = (float) (I % GRID_WIDTH - (GRID_WIDTH / 2));
        g_Grid[I].Z = (float) ((I / GRID_WIDTH) % GRID_WIDTH - (GRID_WIDTH / 2));
        g_Grid[I].Y = (float) ((I / gn_GRID_WIDTH2) % GRID_HEIGHT - (GRID_HEIGHT / 2));
    }
}

void gl_Initialize() {

    InitObject();

    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // Black Background
    glShadeModel(GL_SMOOTH);               // Enables Smooth Color Shading
    glClearDepth(1.0f);                    // Depth Buffer Setup
    glDepthFunc(GL_LESS);                  // The Type Of Depth Test TO Do
    glBlendFunc(GL_ONE, GL_ONE);
 
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // Realy Nice perspective calculations
   
    glEnable(GL_TEXTURE_2D);               // Enable Texture Mapping

    // Create the textures.
    g_mt.resize(3); wstring ws;
    ws = g_sresource; ws.append(L"background.png"); wcscpy_s(g_mt[0].FullName, (WCHAR*) ws.c_str()); g_mt[0].ID = 1; g_mt[0].Square = 0;
    ws = g_sresource; ws.append(L"ball.png"); wcscpy_s(g_mt[1].FullName, (WCHAR*) ws.c_str()); g_mt[1].ID = 2; g_mt[1].Square = 0;
    ws = g_sresource; ws.append(L"bubble.png"); wcscpy_s(g_mt[2].FullName, (WCHAR*) ws.c_str()); g_mt[2].ID = 3; g_mt[2].Square = 0;
    ZI_MakeMultipleTexture(&g_mt[0], (long) UBOUND(g_mt));

    // Initialise all particles
    InitParticle();

    srand((unsigned) time(NULL));
}

// Draws the tail of the g_Particle. Color fades away. Fast particles are red
void DrawParticleTail(IN long ElapsedTime) {
    long I, P; float C;

    for (P = 1; P <= gn_Particles; P++) {

        // Draw the path of the g_Particle and fade the color
        g_Particle[P].Step = 1.0f - (g_Particle[P].DestValue - ElapsedTime) / 10.0f * g_Particle[P].Speed;
        glBegin(GL_LINE_STRIP);
           glColor3f(1, 1, 1);
           glVertex3f((g_Grid[g_Particle[P].NextVertex].X - g_Grid[g_Particle[P].Visited[0]].X) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[0]].X,
                      (g_Grid[g_Particle[P].NextVertex].Y - g_Grid[g_Particle[P].Visited[0]].Y) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[0]].Y,
                      (g_Grid[g_Particle[P].NextVertex].Z - g_Grid[g_Particle[P].Visited[0]].Z) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[0]].Z);
           for (I = 1; I < TAIL_LENGTH; I++) {
               C = (TAIL_LENGTH - I) / (float) TAIL_LENGTH - g_Particle[P].Step / (float) TAIL_LENGTH;
               if (g_Particle[P].Speed < 0.049f) {
                   glColor3f(C, C, 0.5f + C); }
               else {
                   glColor3f(0.5f + C, C, C ); // faster particles are red
               }
               glVertex3fv((GLfloat*) &g_Grid[g_Particle[P].Visited[I - 1]]);
           }
           glColor3f(0.0f, 0.0f, 0.1f);
           glVertex3f((g_Grid[g_Particle[P].Visited[TAIL_LENGTH-2]].X - g_Grid[g_Particle[P].Visited[TAIL_LENGTH-1]].X) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[TAIL_LENGTH-1]].X,
                      (g_Grid[g_Particle[P].Visited[TAIL_LENGTH-2]].Y - g_Grid[g_Particle[P].Visited[TAIL_LENGTH-1]].Y) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[TAIL_LENGTH-1]].Y,
                      (g_Grid[g_Particle[P].Visited[TAIL_LENGTH-2]].Z - g_Grid[g_Particle[P].Visited[TAIL_LENGTH-1]].Z) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[TAIL_LENGTH-1]].Z);
        glEnd();
    }
}

// Check if the g_Particle has reached its destination. if it has, select a new destination
void UpdateParticles(IN long ElapsedTime) { 
    long I = 0, P = 0, nRound = 0;
    float rValue= 0.0f;
    for (P = 1; P <= gn_Particles; P++) {
        if (ElapsedTime >= g_Particle[P].DestValue) { // Check if the g_Particle has reached its destination

           for (I = TAIL_LENGTH - 1; I > 0; I--) { // store current location in visited list and thift the other up
               g_Particle[P].Visited[I] = g_Particle[P].Visited[I-1];
           }
           g_Particle[P].Visited[0] = g_Particle[P].NextVertex;

           // select a new random direction
           do {
              I = (rand() % 6); // RND(0,5)
              switch (I) {
              case 0: // left
                   if ((g_Particle[P].Visited[0] % GRID_WIDTH) > 0) {
                       g_Particle[P].NextVertex = g_Particle[P].Visited[0] - 1;
                   } // left
                   break;
              case 1: // right
                   if ((g_Particle[P].Visited[0] % GRID_WIDTH) < GRID_WIDTH - 1) {
                       g_Particle[P].NextVertex = g_Particle[P].Visited[0] + 1;
                   }
                   break;
              case 2: // back
                   if ((g_Particle[P].Visited[0] % gn_GRID_WIDTH2) > GRID_WIDTH) {
                       g_Particle[P].NextVertex = g_Particle[P].Visited[0] - GRID_WIDTH;
                   }
                   break;
              case 3: // forwad
                   if ((g_Particle[P].Visited[0] % gn_GRID_WIDTH2) < gn_GRID_WIDTH2 - GRID_WIDTH) {
                       g_Particle[P].NextVertex = g_Particle[P].Visited[0] + GRID_WIDTH;
                   }
                   break;
              case 4: // down
                   if ((g_Particle[P].Visited[0] / gn_GRID_WIDTH2) > 0) {
                       g_Particle[P].NextVertex = g_Particle[P].Visited[0] - gn_GRID_WIDTH2;
                   }
                   break;
              case 5: // Up
                   if ((g_Particle[P].Visited[0] / gn_GRID_WIDTH2) < GRID_HEIGHT -1) {
                       g_Particle[P].NextVertex = g_Particle[P].Visited[0] + gn_GRID_WIDTH2;
                   }
                   break;
              }
              if (g_Particle[P].NextVertex != g_Particle[P].Visited[0]) {    // not equal current vertex
                 if (g_Particle[P].NextVertex != g_Particle[P].Visited[1]) { // not equal previous vertex
                    I = -1;
                 }
              }
           } while (I != -1);

           // calculate when the next endpoint will be reached
           rValue = 10 / g_Particle[P].Speed;
           nRound = (long) rValue; nRound = (ElapsedTime / nRound) * 10 + 10;
           g_Particle[P].DestValue = (long) (nRound / g_Particle[P].Speed);
        }
    }
}

// Draws the head of the g_Particle. Fast particles are red
void DrawParticleHead() {
    long P; float X, Y, Z;

    glBindTexture(GL_TEXTURE_2D, g_mt[1].Texture);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);


    for (P = 1; P <= gn_Particles; P++) {
        X =(g_Grid[g_Particle[P].NextVertex].X - g_Grid[g_Particle[P].Visited[0]].X) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[0]].X;
        Y =(g_Grid[g_Particle[P].NextVertex].Y - g_Grid[g_Particle[P].Visited[0]].Y) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[0]].Y;
        Z =(g_Grid[g_Particle[P].NextVertex].Z - g_Grid[g_Particle[P].Visited[0]].Z) * g_Particle[P].Step + g_Grid[g_Particle[P].Visited[0]].Z;
        if (g_Particle[P].Speed < 0.049f) {
           glColor4ub(255,255,255, 255);
           glBindTexture(GL_TEXTURE_2D, g_mt[1].Texture); }
        else {
           glColor4ub(255,255,255, 255); // faster particles are red
           glBindTexture(GL_TEXTURE_2D, g_mt[2].Texture);
        }

        glPushMatrix();
           glTranslatef(X, Y, Z);
           GL_ChartPoint(0.125f, 0.0f);
        glPopMatrix();

    }

    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
}

// This is the place where to customize the OpenGL scene.
void gl_DrawScene() {
    static long nFlip;
    nFlip = !nFlip; if (nFlip == 0) { return; } // Save CPU

    long LastTime; 
    float rAngle;
    static long DemoStart, ElapsedTime;

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   // Clear The Screen And The Depth Buffer
    glLoadIdentity();                                     // Reset The View

    if (DemoStart == 0) { DemoStart = GetTickCount(); }
    LastTime = ElapsedTime;
    ElapsedTime = (long) GetTickCount() - DemoStart;     // Calculate Elapsed Time
    ElapsedTime = (LastTime + ElapsedTime) / 2;          // Average it out FOR smoother movement

    glPushMatrix();
       glEnable(GL_TEXTURE_2D);
       glBindTexture(GL_TEXTURE_2D, g_mt[0].Texture);
       glRotatef(ElapsedTime / 500.0f, 0.0f, 0.0f, 1.0f);
       glColor4ub(255,255,255, 255);
       GL_DrawQuad(99.9f, 99.9f, -99.9f);
       glDisable(GL_TEXTURE_2D);
    glPopMatrix();

    // Get Mousing coordinates, zooming, rotation.
    float rHalfRadius = 0.0f, rChartXoff = 0.0f, rChartYoff = 0.0f, rChartZoff = 0.0f, rChartZoom = 0.0f, rChartXrot = 0.0f, rChartYrot = 0.0f;
    GL_ChartGetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot);
    glTranslatef(rChartXoff, rChartYoff, rChartZoff);
    // Zooming factor.
    glScalef(rChartZoom, rChartZoom, rChartZoom);
    // Mousing rotation
    glRotatef(-rChartYrot, 1, 0, 0);
    glRotatef(-rChartXrot, 0, 1, 0);

    rAngle =  ElapsedTime / 300.0f;
    if (rAngle > (359.98f * 5)) { 
        DemoStart = GetTickCount();
        ElapsedTime = 0; 
        InitObject(); 
        InitParticle();
    }

    glRotatef(rAngle, 0.0f, 1.0f, 0.0f);
    
    // Check if the g_Particle has reached its destination. if so, select a new destination
    UpdateParticles(ElapsedTime);
    
    // Draw the g_Particle tail
    DrawParticleTail(ElapsedTime);
   
    // Draw the actual particles
    DrawParticleHead();

    // Update the display
    ZI_UpdateGLWindow(gh_GLhWnd);

}

LRESULT CALLBACK WndProc(IN HWND hWnd, IN UINT uMsg, IN WPARAM wParam, IN LPARAM lParam) {

    MINMAXINFO* pMM;
    wstring ws;

    switch (uMsg) {

    case WM_GETMINMAXINFO:
         pMM = (MINMAXINFO*) lParam;
         pMM->ptMinTrackSize.x = MinTrackSizeW(0);
         pMM->ptMinTrackSize.y = MinTrackSizeH(0);
         break;

    case WM_TIMER:
         gl_DrawScene();
         return 0;

    case WM_MOUSEWHEEL:
         if (GetFocus() != gh_GLhWnd) { SetFocus(gh_GLhWnd); }
         break;

    case WM_SIZE:
         ZI_ResizeGLWindow(gh_GLhWnd);
         gl_DrawScene();
         break;

    case WM_DESTROY:
         PostQuitMessage(0);
         return 0;
    }
    return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void Animate (IN LPVOID nDelay) {
    DWORD nWait = 0;
    long nFrame;
    for (;;) {
        nFrame = ZD_GetObjectFrameToUse(ID_SPIN01) + 1; if (nFrame > ZD_GetObjectFrameCount(ID_SPIN01)) { nFrame = 1; }
        ZD_SetObjectFrameToUse(ID_SPIN01, nFrame, TRUE);
        Sleep((long) nDelay);
    }
}

long StartAnimation (IN long Delay) {
    long nRet = LB_ERR;
    DWORD dwThreadId = 0;
    HANDLE hThread = CreateThread(NULL,                                  // default security attributes
                                  0,                                     // use default stack size  
                                  (LPTHREAD_START_ROUTINE ) Animate,     // thread function name
                                  (LPVOID) Delay,                        // argument to thread function 
                                  0,                                     // use default creation flags 
                                  &dwThreadId);                          // returns the thread identifier
    if (hThread) { nRet = 0; Sleep(100); }
    CloseHandle(hThread);
    return nRet;
}

int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {

    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);

    MSG msg;
    long nRet = 0;
    WNDCLASSEX wcx = {0};
    WCHAR szWindowClass[] = L"ZPARTICLINES";

    RECT rc; POINT p;
    long x, y, IsInitialized;
    DWORD dwStyle, dwExStyle;
    HWND hMain, hFound, hCtrl;

    ZGLFONT UseFont = {0};

    HANDLE hMutex = CreateMutex(NULL, 0, szWindowClass);
    if (hMutex) {
       if (GetLastError() == ERROR_ALREADY_EXISTS) {
          for(;;){
              hFound = FindWindow(szWindowClass, NULL); if (hFound) { break; }
              while (PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE)) { }
          }
          if (IsIconic(hFound)) { ShowWindow(hFound, SW_RESTORE); }
          SetForegroundWindow(hFound);
          return 0;
       }
    }

    wcx.cbSize = sizeof(wcx);
    IsInitialized = GetClassInfoEx(hInstance, szWindowClass, &wcx);
    if (!IsInitialized) {
       wcx.style         = 0; // CS_HREDRAW | CS_VREDRAW;
       wcx.lpfnWndProc   = WndProc;
       wcx.cbClsExtra    = 0;
       wcx.cbWndExtra    = 0;
       wcx.hInstance     = hInstance;
       wcx.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
       wcx.hCursor       = LoadCursor(NULL, IDC_ARROW);
       wcx.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
       wcx.lpszMenuName  = NULL;
       wcx.lpszClassName = szWindowClass;
       wcx.hIconSm       = wcx.hIcon;
       if (RegisterClassEx(&wcx)) { IsInitialized = -1; }
    }
    if (IsInitialized) {

       // Window Extended Style
       dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
       // Windows Style
       dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
       SetRect(&rc, 0, 0, ClientW, ClientH);
       AdjustWindowRectEx(&rc, dwStyle, FALSE, dwExStyle);    // Adjust Window To True Requested Size

       x = max((GetSystemMetrics(SM_CXSCREEN) - rc.right - rc.left) / 2, 0);
       y = max((GetSystemMetrics(SM_CYSCREEN) - rc.bottom - rc.top) / 2, 0);
       MinTrackSizeW(rc.right - rc.left);
       MinTrackSizeH(rc.bottom - rc.top);

       // Create The Window
       wstring sCaption = L"Particle lines";
       if (sizeof(LONG_PTR) == 8) {
           sCaption += L" 64-bit"; }
       else {
           sCaption += L" 32-bit";
       }

       hMain = CreateWindowEx(dwExStyle,                  // Extended Style For The Window
                             szWindowClass,               // Class Name
                             (WCHAR*) sCaption.c_str(),   // Window Title
                             dwStyle,                     // Defined Window Style
                             x, y,                        // Window Position
                             MinTrackSizeW(0),            // Calculate Window Width 
                             MinTrackSizeH(0),            // Calculate Window Height
                             NULL,                        // No Parent Window
                             NULL,                        // No Menu
                             hInstance,                   // Instance
                             NULL);                       // Dont Pass Anything To WM_CREATE

       if (hMain) {
           // 6.04 Create GDImage OpenGL antialiased control.
           dwStyle = WS_CHILD | WS_VISIBLE;
           dwExStyle = 0;
           gh_GLhWnd = WGL_CreateWindow(dwExStyle, dwStyle, 10, 10, 712, 512, hMain, (HMENU) IDC_CTRL);

           if (gh_GLhWnd) {

               // Anchor the GDImage WGL control (make it a resizable)
               ZI_SetAnchorMode(GetDlgItem(hMain, IDC_CTRL), ANCHOR_HEIGHT_WIDTH);

               // Use GDImage OpenGL automatic mousing support.
               GL_UseMouseManager(gh_GLhWnd);

               g_sresource = EXEpath(); g_sresource.append(L"Resource\\");

               gl_Initialize();

               wstring SkinTheme = EXEpath(); SkinTheme += L"Seven.sks";
               if (skInitEngine ((WCHAR*) SkinTheme.c_str(), L"")) {
                   skSkinWindow(hMain, L"Dock|Undock|Minimize|Maximize|Restore|Close");

                   // Add a skinned border around the GDImage control.
                   GetWindowRect(gh_GLhWnd, &rc);
                   p.x = rc.left; p.y = rc.top;
                   ScreenToClient(hMain, &p);
                   skBorder(gh_GLhWnd, p.x - 1, p.y - 1, rc.right - rc.left + 2, rc.bottom - rc.top + 2, 0, 1);

                   // Add a WinLIFT clock
                   wstring ws = g_sresource; ws.append(L"Clock.png");
                   hCtrl = skClockCtrl(hMain, (WCHAR*) ws.c_str(), p.x + rc.right - rc.left + 28, p.y, 0, 0, IDC_CLOCK, 0x7FCB0000, 0x7F8F8F8F, 0);
                   skSetAnchorCtrl(hCtrl, ANCHOR_RIGHT);
                   skCreateToolTip(hCtrl, L"WinLIFT clock widget");

                   // Add a static control to display the help text.
                   GetClientRect(hMain, &rc);
                   ws = L"\n\rLeft mouse button,\n\rchange view angle.\n\r\n\rRight mouse button,\n\rchange x,y location.\n\r\n\rMouse wheel,\n\rzoom in / out.";
                   hCtrl = CreateWindowEx(0, L"STATIC", (WCHAR*) ws.c_str(), WS_CHILD | WS_VISIBLE | SS_CENTER | WS_BORDER,
                                          rc.right - (129 + 8), 10 + (22 + 5) * 5 + 5, 129, 150, hMain, (HMENU) IDC_LABEL, hInstance, NULL);
                   // Skin the static control on the fly.
                   skSkinChildCtrl(hCtrl, 0);
                   ZI_SetAnchorMode(hCtrl, ANCHOR_RIGHT);
                   // Use a custom font with the static control.
                   skSetLabelFont(hCtrl, L"TREBUCHET MS", 12, skARGB(200, 255,255,255), FontStyleBold);

                   // Diplay the GDImage logo.
                   ws = g_sresource; ws.append(L"GDImage.png");
                   hCtrl = skStaticImage(hMain, (WCHAR*) ws.c_str(), rc.right - (129 + 6), 330, 124, 45, IDC_LOGO);
                   ZI_SetAnchorMode(hCtrl, ANCHOR_BOTTOM_RIGHT);


                   // For the fun we add a little GDImage animation
                   hCtrl = CreateWindowEx(0, GDImageClassName, NULL,  WS_CHILD | WS_VISIBLE,
                                         rc.right - (128 + 8), rc.bottom - (128 + 8), 128, 128,
                                         hMain, (HMENU) IDC_SPINNER, hInstance, NULL);
                   long imgW = 0, imgH= 0, nReserved = 0;
                   ws = g_sresource; ws.append(L"particle128.ski");
                   HBITMAP   hBitmap = skSkiToDib ((WCHAR*) ws.c_str(), nReserved);
                   ZI_GetBitmapSize(hBitmap, imgW, imgH);
                   ZD_DrawBitmapToCtrl(hCtrl, (128 - imgH) / 2, (128 - imgH) / 2, hBitmap, ZD_ColorARGB(255, 0), ID_SPIN01, ZS_VISIBLE);
                   // Use the WinLIFT background for better illusion.
                   ZI_UseWinLIFTbackground(hCtrl, TRUE, TRUE);
                   ZD_SetObjectFrameCount(ID_SPIN01, (BYTE) (imgW / imgH));
                   ZD_SetObjectFrameToUse(ID_SPIN01, 1, FALSE);
                   skSetAnchorCtrl(GetDlgItem(hMain, IDC_SPINNER), ANCHOR_BOTTOM_RIGHT); 
                   // Lock and disable the sprite image.
                   ZD_SetObjectLocked(ID_SPIN01, TRUE);


                   // Show the main window
                   ShowWindow(hMain, nCmdShow);
                   SetForegroundWindow(hMain);                  // Slightly Higher Priority
                   SetFocus(hMain);                             // Sets Keyboard Focus To The Window

                   // Main message pump.
                   SetTimer(hMain, 1, 0, 0);   // 66 FPS
                   StartAnimation(33);
                   while (GetMessage(&msg, NULL, 0, 0)) {
                        TranslateMessage(&msg);
                        DispatchMessage(&msg);
                   }
                   nRet = (long) msg.wParam;
                   KillTimer(hMain, 1);
               }
               // Free OpenGL resources.
               ZI_DestroyTexture(&g_mt[0], (long) UBOUND(g_mt));
           }
       }
    }

    if (hMutex) { CloseHandle(hMutex); }
    return nRet;
}
